import os
import urllib


def convert_to_boolean(string):
    """
    Shipyard can't support passing Booleans to code, so we have to convert
    string values to their boolean values.
    """
    if string in ['True', 'true', 'TRUE']:
        value = True
    else:
        value = False
    return value


def set_environment_variables(environment_variables_to_set):
    """
    Set any args.value to an environment variable.
    This will override system defaults.
    Must be formatted as:
    {'ENVIRONMENT_VARIABLE_NAME': args.value}
    """
    for environment_variable_name, arg_name in environment_variables_to_set.items():
        os.environ[environment_variable_name] = arg_name
    return


def create_shipyard_link():
    """
    Create a link back to the Shipyard log page for the currently running voyage.
    """
    org_name = os.environ.get('SHIPYARD_ORG_NAME')
    project_id = os.environ.get('SHIPYARD_PROJECT_ID')
    fleet_id = os.environ.get('SHIPYARD_FLEET_ID')
    vessel_id = os.environ.get('SHIPYARD_VESSEL_ID')
    fleet_log_id = os.environ.get('SHIPYARD_FLEET_LOG_ID')
    vessel_log_id = os.environ.get('SHIPYARD_LOG_ID')

    if project_id and fleet_id and fleet_log_id:
        dynamic_link_section = urllib.parse.quote(
            f'{org_name}/projects/{project_id}/fleets/{fleet_id}/logs/{fleet_log_id}/{vessel_log_id}')
        shipyard_link = f'https://app.shipyardapp.com/{dynamic_link_section}'
    elif project_id and vessel_id and log_id:
        dynamic_link_section = urllib.parse.quote(
            f'{org_name}/projects/{project_id}/vessels/{vessel_id}/logs/{vessel_log_id}')
        shipyard_link = f'https://app.shipyardapp.com/{dynamic_link_section}'
    else:
        shipyard_link = 'https://www.shipyardapp.com'
    return shipyard_link



import os
import pickle
from . import files


def determine_base_artifact_folder(vendor):
    """
    Creates the base folder structure for storing artifacts.
    Uses USER to support local runs.
    """
    artifact_folder_default = f'{os.environ.get("USER")}-artifacts'
    base_artifact_folder = files.clean_folder_name(
        f'{os.environ.get("SHIPYARD_ARTIFACTS_DIRECTORY",artifact_folder_default)}/{vendor}-blueprints/')
    return base_artifact_folder


def determine_artifact_subfolders(base_artifact_folder):
    subfolders = ['artifacts', 'responses', 'logs', 'variables']
    subfolder_paths = {}
    for subfolder in subfolders:
        clean_subfolder_path = files.clean_folder_name(
            f'{base_artifact_folder}/{subfolder}')
        subfolder_paths[subfolder] = clean_subfolder_path
    return subfolder_paths


def create_artifacts_folders(subfolder_paths):
    """
    Creates all artifact subfolders so that generated logging files can be inserted in immediately.
    Returns a dictionary with the paths listed out.
    """
    for key, path in subfolder_paths.items():
        files.create_folder_if_dne(path)


def create_pickle_file(subfolder_paths, variable_name, variable_value):
    pickle_file_name = files.combine_folder_and_file_name(
        subfolder_paths['variables'], f'{variable_name}.pickle')
    with open(pickle_file_name, 'wb') as f:
        pickle.dump(variable_value, f)


def read_pickle_file(subfolder_paths, variable_name):
    pickle_file_name = files.combine_folder_and_file_name(
        subfolder_paths['variables'], f'{variable_name}.pickle')
    with open(pickle_file_name, 'rb') as f:
        variable_name = pickle.load(f)
    return variable_name



import hashlib


def hash_text(text_var, hash_algorithm='sha256'):
    """
    Hash the provided text with a specified hash_algorithm.
    """
    if hash_algorithm == 'sha256':
        hashed_text = hashlib.sha256(text_var.encode('ascii')).hexdigest()
    if hash_algorithm == 'sha512':
        hashed_text = hashlib.sha512(text_var.encode('ascii')).hexdigest()
    if hash_algorithm == 'md5':
        hashed_text = hashlib.md5(text_var.encode('ascii')).hexdigest()
    return hashed_text



from dateutil import parser
import os


def extract_from_timestamp(timestamp, time_part):
    """
    Extract the year, month, day, hour, minute, or second from a Shipyard generated timestamp.
    """
    datetime_timestamp = parser.parse(timestamp)
    if time_part in [
        'year',
        'YEAR',
        'Year',
        'Y',
        'YY',
        'YYYY',
        'y',
        'yy',
        'yyyy'
    ]:
        time_value = datetime_timestamp.year
    if time_part in ['month', 'MONTH', 'Month', 'M', 'MM']:
        time_value = datetime_timestamp.month
    if time_part in ['day', 'DAY', 'Day', 'D', 'DD', 'd', 'dd']:
        time_value = datetime_timestamp.day
    if time_part in [
        'hour',
        'HOUR',
        'Hour',
        'H',
        'HH',
        'h',
        'hh',
        'hr',
        'HR',
        'Hr'
    ]:
        time_value = datetime_timestamp.hour
    if time_part in [
        'minute',
        'MINUTE',
        'Minute',
        'm',
        'mm',
        'min',
        'MIN',
        'Min'
    ]:
        time_value = datetime_timestamp.minute
    if time_part in [
        'second',
        'SECOND',
        'Second',
        'S',
        'SS',
        's',
        'ss',
        'sec',
        'SEC',
        'Sec'
    ]:
        time_value = datetime_timestamp.second
    return str(time_value)


def set_extra_time_environment_variables():
    """
    Expand the existing set of Shipyard timestamp environment variables to include
    year, month, day, hour, minute, and second.
    """
    shipyard_elements = ['VESSEL', 'FLEET']
    time_types = ['START', 'SCHEDULED']

    for element in shipyard_elements:
        for time_type in time_types:
            datetime_object = parser.parse(
                os.environ.get(f'SHIPYARD_{element}_{time_type}_TIME'))
            os.environ[f'SHIPYARD_{element}_{time_type}_TIME_YEAR'] = str(
                datetime_object.year)
            os.environ[f'SHIPYARD_{element}_{time_type}_TIME_MONTH'] = str(
                datetime_object.month)
            os.environ[f'SHIPYARD_{element}_{time_type}_TIME_DAY'] = str(
                datetime_object.day)
            os.environ[f'SHIPYARD_{element}_{time_type}_TIME_HOUR'] = str(
                datetime_object.hour)
            os.environ[f'SHIPYARD_{element}_{time_type}_TIME_MINUTE'] = str(
                datetime_object.minute)
            os.environ[f'SHIPYARD_{element}_{time_type}_TIME_SECOND'] = str(
                datetime_object.second)



import os
import re
import glob
import json
from zipfile import ZipFile
import tarfile

# Functions for Files


def extract_file_name_from_source_full_path(source_full_path):
    """
    Use the file name provided in the source_file_name variable. Should be run only
    if a destination_file_name is not provided.
    """
    destination_file_name = os.path.basename(source_full_path)
    return destination_file_name


def enumerate_destination_file_name(destination_file_name, file_number=1):
    """
    Append a number to the end of the provided destination file name, before the file extension.
    Only used when multiple files are matched to, preventing the destination file from being continuously overwritten.
    """
    if re.search(r'\.', destination_file_name):
        destination_file_name = re.sub(
            r'\.', f'_{file_number}.', destination_file_name, 1)
    else:
        destination_file_name = f'{destination_file_name}_{file_number}'
    return destination_file_name


def determine_destination_file_name(
    *,
    source_full_path,
    destination_file_name,
        file_number=None):
    """
    Determine what the destination_file_name should be.
    If the destination_file_name was provided use that name.
    If the destination_file_name is not provided, use the original source_file_name.
    If multiple files will be created, add file enumeration to the chosen method.
    """
    if destination_file_name:
        if file_number:
            destination_file_name = enumerate_destination_file_name(
                destination_file_name, file_number)
        else:
            destination_file_name = destination_file_name
    else:
        destination_file_name = extract_file_name_from_source_full_path(
            source_full_path)

    return destination_file_name

# Functions for Folders


def clean_folder_name(folder_name):
    """
    Cleans folder names by removing duplicate '/' as well as leading and trailing '/' characters.
    """
    folder_name = folder_name.strip('/')
    if folder_name != '':
        folder_name = os.path.normpath(folder_name)
    return folder_name


def create_folder_if_dne(destination_folder_name):
    """
    Checks to verify if the provided folder already exists.
    If not, the folder and all subfolders are created.
    """
    if not os.path.exists(destination_folder_name) and (
            destination_folder_name != ''):
        os.makedirs(destination_folder_name)

# Functions for mixing File + Folder


def combine_folder_and_file_name(folder_name, file_name):
    """
    Combine together the provided folder_name and file_name into one path variable.
    """
    combined_name = os.path.normpath(
        f'{folder_name}{"/" if folder_name else ""}{file_name}')
    combined_name = os.path.normpath(combined_name)
    return combined_name


def determine_destination_full_path(
        destination_folder_name,
        destination_file_name,
        source_full_path,
        file_number=None):
    """
    Determine the full destination path of the file to be uploaded or downloaded.
    """
    destination_file_name = determine_destination_file_name(
        destination_file_name=destination_file_name,
        source_full_path=source_full_path,
        file_number=file_number)
    destination_full_path = combine_folder_and_file_name(
        destination_folder_name, destination_file_name)
    return destination_full_path

# Functions for handling large files


def compress_files(file_paths, destination_full_path, compression):
    """
    Given a list of files, compress all of them into a single file.
    Keeps the existing directory structure in tact.
    """
    if f'.{compression}' in destination_full_path:
        compressed_file_name = destination_full_path
    else:
        compressed_file_name = f'{destination_full_path}.{compression}'

    if compression == 'zip':
        compress_with_zip(file_paths, compressed_file_name, compression)

    if 'tar' in compression:
        compress_with_tar(file_paths, compressed_file_name, compression)

    return compressed_file_name


def compress_with_zip(file_paths, compressed_file_name, compression):
    """
    Compress a list of files using the zip method.
    """
    write_method = determine_write_method(compression)

    with ZipFile(compressed_file_name, write_method) as zip:
        for file in file_paths:
            file = clean_folder_name(file.replace(os.getcwd(), ''))
            zip.write(file)
            print(
                f'Successfully compressed {file} into {compressed_file_name}')


def compress_with_tar(file_paths, compressed_file_name, compression):
    """
    Compress a list of files using the tar method.
    """
    write_method = determine_write_method(compression)

    with tarfile.open(compressed_file_name, write_method) as tar:
        for file in file_paths:
            file = clean_folder_name(file.replace(os.getcwd(), ''))
            tar.add(file)
            print(
                f'Successfully compressed {file} into {compressed_file_name}')


def determine_write_method(compression):
    """
    Given a specified compression type, choose the write method
    for generating the file.
    """
    if compression == 'tar.bz2':
        write_method = 'w:bz2'
    if compression == 'tar.gz':
        write_method = 'w:gz'
    else:
        write_method = 'w'

    return write_method


def decompress_file(source_full_path, destination_full_path, compression):
    """
    Decompress a given file, using the specified compression method.
    """

    if compression == 'zip':
        decompress_with_zip(
            source_full_path,
            destination_full_path,
            compression)

    if 'tar' in compression:
        decompress_with_tar(
            source_full_path,
            destination_full_path,
            compression)


def decompress_with_zip(source_full_path, destination_full_path, compression):
    read_method = determine_read_method(compression)
    with ZipFile(source_full_path, read_method) as zip:
        zip.extractall(destination_full_path)
        print(
            f'Successfully extracted files from {source_full_path} to {destination_full_path}')


def decompress_with_tar(source_full_path, destination_full_path, compression):
    read_method = determine_read_method(compression)
    file = tarfile.open(source_full_path, read_method)
    file.extractall(path=destination_full_path)
    print(
        f'Successfully extracted files from {source_full_path} to {destination_full_path}')


def determine_read_method(compression):
    """
    Given a specified compression type, choose the read method
    for opening the file.
    """
    if compression == 'tar.bz2':
        read_method = 'r:bz2'
    if compression == 'tar.gz':
        read_method = 'r:gz'
    else:
        read_method = 'r'

    return read_method


def is_file_too_large(file_path, max_size_bytes):
    """
    Determine if the file is too large for a specified limit.
    Used to conditionally compress a file.
    """
    if os.stat(file_path).st_size >= max_size_bytes:
        return True
    else:
        return False


def are_files_too_large(file_paths, max_size_bytes):
    """
    Determine if the total size of all files in a list are too large for a specified limit.
    Used to conditionally compress a file.
    """
    total_size = 0
    for file in file_paths:
        total_size += os.stat(file).st_size
    if total_size >= max_size_bytes:
        return True
    else:
        return False

# Functions for Regex Matching Logic


def find_all_local_file_names(source_folder_name=None):
    """
    Returns a list of all files that exist in the current working directory,
    filtered by source_folder_name if provided.
    """
    cwd = os.getcwd()
    cwd_extension = os.path.normpath(f'{cwd}/{source_folder_name}/**')
    file_names = glob.glob(cwd_extension, recursive=True)
    return file_names


def find_all_file_matches(file_names, file_name_re):
    """
    Return a list of all matching_file_names that matched the regular expression.
    """
    matching_file_names = []
    for file in file_names:
        if re.search(file_name_re, file):
            matching_file_names.append(file)
    print(f'Found {len(matching_file_names)} file matches.')
    print(matching_file_names)
    return matching_file_names

# Functions for Writing Files


def write_json_to_file(json_object, file_name):
    with open(file_name, 'w') as f:
        f.write(
            json.dumps(
                json_object,
                ensure_ascii=False,
                indent=4))
    print(f'JSON data stored at {file_name}')
    return


def write_text_to_file(text_var, file_name):
    with open(file_name, 'w') as f:
        f.write(text_var)
    print(f'Text data stored at {file_name}')
    return



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['shipyard_utils']

package_data = \
{'': ['*'], 'shipyard_utils': ['blake-artifacts/vvv-blueprints/variables/*']}

install_requires = \
['python-dateutil==2.8.2']

setup_kwargs = {
    'name': 'shipyard-utils',
    'version': '0.1.3',
    'description': 'Small scripts and utilities to make Shipyard Blueprint development easier.',
    'long_description': None,
    'author': 'Blake Burch',
    'author_email': 'blake@shipyardapp.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
